@@ -17,54 +17,63 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.h 672 2006-08-05 12:25:49Z ntrel $
+ * $Id: filetypes.h 1038 2006-11-30 15:42:52Z ntrel $
  */
 
 
 #ifndef GEANY_FILETYPES_H
 #define GEANY_FILETYPES_H 1
 
+#ifndef PLAT_GTK
+#   define PLAT_GTK 1	// needed for ScintillaWidget.h
+#endif
+
+#include "Scintilla.h"
+#include "ScintillaWidget.h"
+
 
 enum
 {
-	GEANY_FILETYPES_C = 0,		// 0
-	GEANY_FILETYPES_CPP,		// 1
-	GEANY_FILETYPES_D,			// 2
-	GEANY_FILETYPES_JAVA,		// 3
-	GEANY_FILETYPES_PASCAL,		// 4
-	GEANY_FILETYPES_ASM,		// 5
-	GEANY_FILETYPES_CAML,		// 6
-	GEANY_FILETYPES_PERL,		// 7
-	GEANY_FILETYPES_PHP,		// 8
-	GEANY_FILETYPES_PYTHON,		// 9
-	GEANY_FILETYPES_RUBY,		// 10
-	GEANY_FILETYPES_TCL,		// 11
-	GEANY_FILETYPES_SH,			// 12
-	GEANY_FILETYPES_MAKE,		// 13
- 	GEANY_FILETYPES_XML,		// 14
-	GEANY_FILETYPES_DOCBOOK,	// 15
-/*
-	GEANY_FILETYPES_HTML,		// 16
-*/
-	GEANY_FILETYPES_CSS,		// 17
-	GEANY_FILETYPES_SQL,		// 18
-	GEANY_FILETYPES_LATEX,		// 19
-	GEANY_FILETYPES_OMS,		// 20
-	GEANY_FILETYPES_CONF,		// 21
-	GEANY_FILETYPES_ALL,		// 22
-	GEANY_MAX_FILE_TYPES		// 23
+	GEANY_FILETYPES_C = 0,
+	GEANY_FILETYPES_CPP,
+	GEANY_FILETYPES_D,
+	GEANY_FILETYPES_JAVA,
+	GEANY_FILETYPES_PASCAL,
+	GEANY_FILETYPES_ASM,
+	GEANY_FILETYPES_FORTRAN,
+	GEANY_FILETYPES_CAML,
+	GEANY_FILETYPES_PERL,
+	GEANY_FILETYPES_PHP,
+	GEANY_FILETYPES_PYTHON,
+	GEANY_FILETYPES_RUBY,
+	GEANY_FILETYPES_TCL,
+	GEANY_FILETYPES_FERITE,
+	GEANY_FILETYPES_SH,
+	GEANY_FILETYPES_MAKE,
+ 	GEANY_FILETYPES_XML,
+	GEANY_FILETYPES_DOCBOOK,
+	GEANY_FILETYPES_HTML,
+	GEANY_FILETYPES_CSS,
+	GEANY_FILETYPES_SQL,
+	GEANY_FILETYPES_LATEX,
+	GEANY_FILETYPES_OMS,
+	GEANY_FILETYPES_VHDL,
+	GEANY_FILETYPES_DIFF,
+	GEANY_FILETYPES_CONF,
+	GEANY_FILETYPES_ALL,	// must be last filetype
+	GEANY_MAX_FILE_TYPES
 };
 
+// Safe wrapper to get the id field of a possibly NULL filetype pointer.
+#define FILETYPE_ID(filetype_ptr) \
+	(((filetype_ptr) != NULL) ? (filetype_ptr)->id : GEANY_FILETYPES_ALL)
 
-struct build_menu_items
+
+struct build_actions
 {
-	GtkWidget		*menu;
-	GtkWidget		*item_compile;
-	GtkWidget		*item_link;
-	GtkWidget		*item_exec;
-	gboolean		 can_compile;
-	gboolean		 can_link;
-	gboolean		 can_exec;
+	gboolean	can_compile:1;
+	gboolean	can_link:1;
+	gboolean	can_exec:1;
 };
 
 struct build_programs
@@ -73,6 +82,7 @@ struct build_programs
 	gchar *linker;
 	gchar *run_cmd;
 	gchar *run_cmd2;
+	gboolean modified;
 };
 
 typedef struct filetype
@@ -90,8 +100,8 @@ typedef struct filetype
 	gchar	 		 *comment_open;
 	gchar	 		 *comment_close;
 	gboolean  		  comment_use_indent;
-	struct build_programs *programs;
-	struct build_menu_items *menu_items;
+	struct build_programs	*programs;
+	struct build_actions	*actions;
 	void (*style_func_ptr) (ScintillaObject*);
 } filetype;
 
@@ -103,18 +113,22 @@ filetype *filetypes_get_from_uid(gint uid);
 
 /* inits the filetype array and fill it with the known filetypes
  * and create the filetype menu*/
-void filetypes_init_types(void);
+void filetypes_init_types();
 
 /* simple filetype selection based on the filename extension */
-filetype *filetypes_get_from_filename(const gchar *filename);
+filetype *filetypes_get_from_filename(gint idx);
 
 /* frees the array and all related pointers */
-void filetypes_free_types(void);
-
-gchar *filetypes_get_template(filetype *ft);
+void filetypes_free_types();
 
 void filetypes_get_config(GKeyFile *config, GKeyFile *configh, gint ft);
 
+void filetypes_save_commands();
+
 void filetypes_select_radio_item(const filetype *ft);
 
+GtkFileFilter *filetypes_create_file_filter(filetype *ft);
+
+gchar *filetypes_get_conf_extension(gint filetype_idx);
+
 #endif
