@@ -1,6 +1,7 @@
 /*
  *      notebook.h - this file is part of Geany, a fast and lightweight IDE
  *
+ *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
  *      Copyright 2006 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: notebook.h 479 2006-06-25 12:13:35Z ntrel $
+ * $Id: notebook.h 1053 2006-12-05 10:37:36Z ntrel $
  */
 
 #ifndef GEANY_NOTEBOOK_H
@@ -25,7 +26,10 @@
 
 void notebook_init();
 
-/* Returns index of notebook page, or -1 on error */
-gint notebook_new_tab(gint doc_idx, gchar *title, GtkWidget *page);
+/* Returns page number of notebook page, or -1 on error */
+gint notebook_new_tab(gint doc_idx);
+
+// Always use this instead of gtk_notebook_remove_page().
+void notebook_remove_page(gint page_num);
 
 #endif
