@@ -17,19 +17,52 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: vte.h 527 2006-07-02 18:34:03Z eht16 $
+ * $Id: vte.h 1078 2006-12-10 21:29:04Z eht16 $
  */
 
 
 #ifndef GEANY_VTE_H
 #define GEANY_VTE_H 1
 
+#ifdef HAVE_VTE
+
 /* include stdlib.h AND unistd.h, because on GNU/Linux pid_t seems to be
  * in stdlib.h, on FreeBSD in unistd.h */
 #include <stdlib.h>
 #include <unistd.h>
 
 
+typedef struct
+{
+	gboolean load_vte;
+	gboolean have_vte;
+	gchar	*lib_vte;
+	gchar	*dir;
+} VteInfo;
+
+extern VteInfo vte_info;
+
+
+typedef struct
+{
+	GtkWidget *vte;
+	GtkWidget *menu;
+	GtkWidget *im_submenu;
+	gboolean scroll_on_key;
+	gboolean scroll_on_out;
+	gboolean ignore_menu_bar_accel;
+	gboolean follow_path;
+	gboolean run_in_vte;
+	gint scrollback_lines;
+	gchar *emulation;
+	gchar *shell;
+	gchar *font;
+	GdkColor *colour_fore;
+	GdkColor *colour_back;
+} VteConfig;
+VteConfig *vc;
+
+
 void vte_init(void);
 
 void vte_close(void);
@@ -38,6 +71,12 @@ void vte_apply_user_settings(void);
 
 void vte_send_cmd(const gchar *cmd);
 
+const gchar* vte_get_working_directory(void);
+
+void vte_cwd(const gchar *filename, gboolean force);
+
+void vte_append_preferences_tab();
+
 /*
 void vte_drag_data_received(GtkWidget *widget, GdkDragContext  *drag_context, gint x, gint y,
 							GtkSelectionData *data, guint info, guint time);
@@ -64,25 +103,9 @@ struct _VteTerminal
 };
 
 
-struct vte_conf
-{
-	GtkWidget *vte;	// small hack, to have always a reference to the widget
-	GtkWidget *menu;
-	gboolean scroll_on_key;
-	gboolean scroll_on_out;
-	gboolean follow_path;
-	gint scrollback_lines;
-	gchar *emulation;
-	gchar *font;
-	GdkColor *color_fore;
-	GdkColor *color_back;
-};
-struct vte_conf *vc;
-
-
 /* store function pointers in a struct to avoid a strange segfault if they are stored directly
  * if accessed directly, gdb says the segfault arrives at old_tab_width(prefs.c), don't ask me */
-struct vte_funcs
+struct VteFunctions
 {
 	GtkWidget* (*vte_terminal_new) (void);
 	pid_t (*vte_terminal_fork_command) (VteTerminal *terminal, const char *command, char **argv,
@@ -106,7 +129,9 @@ struct vte_funcs
 	void (*vte_terminal_set_color_foreground) (VteTerminal *terminal, const GdkColor *foreground);
 	void (*vte_terminal_set_color_background) (VteTerminal *terminal, const GdkColor *background);
 	void (*vte_terminal_feed_child) (VteTerminal *terminal, const char *data, glong length);
+	void (*vte_terminal_im_append_menuitems) (VteTerminal *terminal, GtkMenuShell *menushell);
 };
 
+#endif
 
 #endif
